<?php
namespace app\cms\controller;

use app\common\controller\Front;

class Search extends Front
{

    public function _initialize()
    {
        parent::_initialize();
    }
    
    //站内搜索
    public function index()
    {
        //post请求
        if( $this->request->isPost() ){
            $this->redirect(cmsUrlSearch(['searchText'=>input('post.searchText/s')],'cms','search','index'),302);
        }
        
        //判断是否有关键词
        if( !$this->query['searchText'] ){
            $this->error(lang('error_search_empty'),'cms/index/index');
        }
        
        //请求限制
        if( !$this->searchRequest() ){
            $this->error(lang('error_search_rest'),'cms/index/index');
        }
        
        //地址栏参数
        $info = cmsUrlParams('info_id');
        
        //分页路径
        $info['path'] = cmsUrlSearch([
            'searchText' => $info['search'],
            'pageNumber' => '[PAGE]',
            'pageSize'   => $info['limit'],
            'sortName'   => $info['sort'],
            'sortOrder'  => $info['order'],
        ],'cms','search','index');
        
        //所有搜索引擎链接
        $info = $this->searchUrls($info);
        
        //处理SEO优化
        $info = $this->searchSeo($info);
        
        //查询数据
        $item = articleSelect([
            'cache'   => true,
            'status'  =>'normal',
            'sort'    => $info['sort'],
            'order'   => $info['order'],
            'limit'   => $info['limit'],
            'page'    => $info['page'],
            'search'  => $info['search'],
            /*'whereOr' => ['info_name'=>['like','%'.$info['search'].'%']],
            'meta_query'  => [
                ['key' => ['eq','url_web'], 'value' => ['like','%'.$info['search'].'%']],
            ],*/
        ]);
        
        $this->assign($info);
        
        $this->assign($item);
        
        return $this->fetch();
    }
    
    public function baidu()
    {
        $this->redirect('https://www.baidu.com/s?tn=bds&cl=3&ct=2097152&si='.config('common.site_domain').'&wd='.$this->searchText(),302);
    }
    
    public function sogou()
    {
        $this->redirect('https://www.sogou.com/web?query='.$this->searchText(),302);
    }
    
    public function toutiao()
    {
        $this->redirect('https://so.toutiao.com/search?mod=website&keyword='.$this->searchText(),302);
    }
    
    public function bing()
    {
        $this->redirect('https://cn.bing.com/search?q='.$this->searchText(),302);
    }
    
    public function so()
    {
        $this->redirect('https://www.so.com/s?q='.$this->searchText(),302);
    }
    
    //获取关键字
    private function searchText()
    {
        if( $this->request->isPost() ){
            return DcHtml(input('post.searchText/s','daicuo'));
        }
        return DcHtml(input('searchText/s','daicuo'));
    }
    
    
    //搜索请求是否合法（间隔时长与最大次数）
    private function searchRequest()
    {
        //搜索限制白名单
        if( array_intersect(['administrator','edit'], $this->site['user']['user_capabilities']) ){
            return true;
        }
        //客户端k唯一标识 $_SERVER["HTTP_REFERER"]
        $searchClient = md5($this->request->ip().$this->request->header('user-agent'));
        //60秒内最大搜索次数
        if( config('cms.search_max') ){
            $searchMax = intval(DcCache('cmsmax'.$searchClient));
            if($searchMax > config('cms.search_max')){
                return false;
            }
            DcCache('cmsmax'.$searchClient, $searchMax+1, 60);
        }
        //几秒内不得再次搜索
        if(config('cms.search_interval')){
            if( DcCache('cmssearch'.$searchClient) ){
                return false;
            }
            DcCache('cmssearch'.$searchClient, 1, config('cms.search_interval'));
        }
        return true;
    }
    
    //处理SEO
    private function searchSeo($info)
    {
        $info['search_title'] = str_replace('[searchText]', $info['search'], cmsSeo(config('cms.search_title')));
        $info['search_keywords'] = str_replace('[searchText]', $info['search'], cmsSeo(config('cms.search_keywords')));
        $info['search_description'] = str_replace('[searchText]', $info['search'], cmsSeo(config('cms.search_description')));
        return $info;
    }
    
    //其它搜索引擎路径
    private function searchUrls($info)
    {
        $info['search_urls'] =  [
            'cms'     => cmsUrlSearch(['searchText' => $info['search']], 'cms', 'search', 'index'),
            'baidu'   => cmsUrlSearch(['searchText' => $info['search']], 'cms', 'search', 'baidu'),
            'sogou'   => cmsUrlSearch(['searchText' => $info['search']], 'cms', 'search', 'sogou'),
            'toutiao' => cmsUrlSearch(['searchText' => $info['search']], 'cms', 'search', 'toutiao'),
            'bing'    => cmsUrlSearch(['searchText' => $info['search']], 'cms', 'search', 'bing'),
            'so'      => cmsUrlSearch(['searchText' => $info['search']], 'cms', 'search', 'so'),
        ];
        
        //预留钩子
        \think\Hook::listen('cms_search_urls', $info);
        
        return $info;
    }
}